﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ExtensionMethods;
using System.Security.Cryptography;

namespace BibliothequeClassesSDDetGestion
{
    [Serializable]
    public abstract class ElementArborescence
    {
        // attributs
        private string _nom;
        private DateTime _dateEtHeureCreation;
        private DateTime _dateEtHeureModification;
        private Dossier _dossierParent;       

        public string Nom { get { return _nom; } set { _nom = value; } }
        public DateTime DateEtHeureCreation { get { return _dateEtHeureCreation; } set { _dateEtHeureCreation = value; } }
        public DateTime DateEtHeureModification { get { return _dateEtHeureModification; } set { _dateEtHeureModification = value; } }

        [System.Xml.Serialization.XmlIgnore]
        public Dossier DossierParent { get { return _dossierParent; } set { _dossierParent = value; } }





        // méthodes d'affichage
        public void afficherInfosTemporelles()
        {
            Console.Write("Votre " + this.ToString().Split('.')[1].ToLower() + " " + _nom + " a été créé le " + _dateEtHeureCreation + " et la dernière modification remonte au " + _dateEtHeureModification + "\n");
        }



        // méthodes de sérialisation/chiffrement
        public bool serialiserElementArborescenceIntoFichier(string cheminFichierDeSauvegarde, bool trueXMLfalseBIN)
        {
            bool operationNickel = false;

            try
            {
                if (trueXMLfalseBIN)
                {
                    cheminFichierDeSauvegarde += ".xml";
                }
                else
                {
                    cheminFichierDeSauvegarde += ".txt";
                }

                using (System.IO.FileStream file = System.IO.File.Create(cheminFichierDeSauvegarde))
                {
                    if (trueXMLfalseBIN)
                    {
                        System.Xml.Serialization.XmlSerializer writer = new System.Xml.Serialization.XmlSerializer(this.GetType());
                        writer.Serialize(file, this);
                        operationNickel = true;
                    }
                    else
                    {
                        System.Runtime.Serialization.Formatters.Binary.BinaryFormatter writer = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
                        writer.Serialize(file, this);
                        operationNickel = true;
                    }
                }
            }
            catch(System.Exception exception)
            {
                Console.Write($"Echec de la serialisation " + exception + " \n");
            }

            return operationNickel;
        }


        public bool serialiserChiffrerElementArborescenceIntoFichier(string cheminFichierDeSauvegarde, bool trueXMLfalseBIN, string password)
        {
            bool operationNickel = false;

            try
            {
                if (trueXMLfalseBIN)
                {
                    cheminFichierDeSauvegarde += ".xml";
                }
                else
                {
                    cheminFichierDeSauvegarde += ".txt";
                }

                byte[] keyPassword = new byte[16];
                keyPassword = keyPassword.createKeyUsingPassword(password);

                using (Aes aes = Aes.Create())
                {
                    /*
                    byte[] key =
                    {
                            0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
                        0x09, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 112
                    };*/

                    aes.Key = keyPassword;
                    byte[] iv = aes.IV;

                    using (System.IO.FileStream file = System.IO.File.Create(cheminFichierDeSauvegarde))
                    {
                        file.Write(iv, 0, iv.Length);

                        using (CryptoStream cryptoStream = new CryptoStream(file, aes.CreateEncryptor(), CryptoStreamMode.Write))
                        {
                            if(trueXMLfalseBIN)
                            {
                                System.Xml.Serialization.XmlSerializer writer = new System.Xml.Serialization.XmlSerializer(this.GetType());
                                writer.Serialize(cryptoStream, this);
                            }
                            else
                            {
                                System.Runtime.Serialization.Formatters.Binary.BinaryFormatter writer = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
                                writer.Serialize(cryptoStream, this);
                            }

                            operationNickel = true;
                        }
                    }
                }
            }
            catch (System.Exception exception)
            {
                Console.Write($"Echec de la serialisation et du chiffrement. " + exception + " \n");
            }

            return operationNickel;
        }
    }
}
